class LanguageManager:

    programming_language = {'.cpp': 'c++11', '.c': 'c++11', '.py': 'python3',
                            '.java': 'java8', '.hs': 'haskel 7.10', '.sh': 'shell',
                            '.r': 'r', '.js': 'javascript', '.rs': 'rust', '.m': 'octave',
                            '.asm': 'asm32', '.clj': 'clojure', '.cs': 'mono c#'}

    def __init__(self):
        pass
