GRAND_TYPE_PASSWORD = "password"
GRAND_TYPE_CREDENTIALS = "client_credentials"

STEPIK_HOST = "https://stepik.org/"
STEPIK_API_URL = STEPIK_HOST + "api"

LESSONS = STEPIK_API_URL + "/lessons/"
LESSONS_PK = LESSONS + "{}"

SUBMISSIONS = STEPIK_API_URL + "/submissions/"
SUBMISSIONS_PK = SUBMISSIONS + "{}"

STEPS = STEPIK_API_URL + "/steps/"
STEPS_PK = STEPS + "{}"

COURSES = STEPIK_API_URL + "/courses/"
COURSES_PK = COURSES + "{}"

ATTEMPTS = STEPIK_API_URL + "/attempts"

SECTIONS = STEPIK_API_URL + "/sections/"
SECTIONS_PK = SECTIONS + "{}"

UNITS = STEPIK_API_URL + "/units/"
