import os 

APP_FOLDER = os.path.dirname(os.path.realpath(__file__))+"/.stepik"
COURSE_CACHE_FILE = APP_FOLDER + "/course_cache_file"
CLIENT_FILE = APP_FOLDER + "/client_file"
ATTEMPT_FILE = APP_FOLDER + "/attempt_file"
CLIENT_ID = ""
CLIENT_SECRET = ""
